#include "adc.h"
#include "../buzzer/buzzer.h"


/* Valoarea citita ajustata */
float adjust_value = 0;
int counter = 0;
double accumulator = 0;
double saved_acc = 0;
int boolean = 0;

void print_value()
{
	char buf[19];
	char ftoa_result[6];

	/*Convert double to string, and print it on the display*/
	ftoa(adjust_value, ftoa_result, 2);
	snprintf(buf, 30, "Alcool:%s mg/L", ftoa_result);
	LCD_printAt(0, buf);

	/*Increment counter to signal main function when to send data to the web server*/
	counter++;

	/*If counter reached the limit*/
	if(counter % 50 == 0){
		/*Make an avarage of the last 50 readings*/
		accumulator = accumulator / 50;

		/*Save the average value into global shared variable*/
		saved_acc = accumulator;

		/*Signal main function to start sending data to the web server*/
		boolean++;

		/*Reset acumulator*/
		accumulator = 0;
	}
}

 /*Intreruperea este triggered cand o conversie este finalizata.*/
ISR(ADC_vect)
{
	unsigned int ADC_value;
	adjust_value = 0.0f;


	/*store new result*/
	ADC_value = ADC;

	/*Convert read value, and bring it to mg/l*/
	adjust_value = ((float) ADC_value * 0.15) / 360;

	/*Add the value to the accumulator*/
	accumulator += adjust_value;

	/*Print the previosly read value*/
	print_value();

	/*Turn on leds based on the read value*/
	blink_LEDs();

	/*Verify if drunk, and play sound*/
	if(!(PINA & (1 << PA1))) {
		speaker_morse(MORSE_SCURT);
	}
}

ISR(TIMER1_COMPA_vect)
{
	/*start an ADC conversion*/
	ADCSRA |= (1 << ADSC);

}

void ADC_init(void)
{
	/*enable ADC with interrupt*/
	ADMUX = (1 << REFS0) | ADC_INIT_CHANNEL;
	/*Prescaler 128*/
	ADCSRA = (1 << ADEN) |
			(1 << ADIE)  |
			(1 << ADPS2) |
			(1 << ADPS1) |
			(1 << ADPS0);
}

void TIMER1_init()
{
/*
     initialize timer to trigger COMPA every 100ms
*/
	TCCR1B = (5 << CS10) | (1 << WGM12);
	TIMSK1 |= (1 << OCIE1A);
	OCR1A = 1561;
}

void prepare_and_start_sensor()
{
	/*Setez PA0 ca intrare pentru semnalul analog*/
	DDRA &= ~(1 << PA0);

	/*Setez pinul PA1 ca intrare + rezistenta de pullup*/
	DDRA &= ~(1 << PA1);
	PORTA |= (1 << PA1);

	/*Init ADC.*/
	ADC_init();

	/*Init Timer1*/
	TIMER1_init();
}
